#
# vmware_best_fit_least_utilized_in_cluster.rb
#
# Description: This method is used to find hosts, datastores in the cluster that are the least utilized
#
# jcutter@redhat.com March 2017.
#

begin
  @debug = true
  $evm.log("info", "Args:    #{MIQ_ARGS.inspect}")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug


  # Get variables
  prov = $evm.root["miq_provision"]
  vm = prov.vm_template
  raise "VM not specified" if vm.nil?
  ems = vm.ext_management_system
  raise "EMS not found for VM [#{vm.name}]" if ems.nil?

  ws_values = prov.options.fetch(:ws_values, nil)
  $evm.log(:info, "ws_values: #{ws_values.inspect}") if @debug
  cluster_name = ws_values[:cluster_name]
  $evm.log(:info, "cluster_name: #{cluster_name}") if @debug
  vm_name = prov.get_option(:vm_name)
  $evm.log(:info, "vm_name: #{vm_name}") if @debug
  cluster = $evm.vmdb(:ems_cluster).find_by_name(cluster_name)
  raise "cluster #{cluster_name} not found!" if cluster.blank?
  $evm.log(:info, "cluster: #{cluster.name}") if @debug
  tags = prov.get_tags
  $evm.log(:info, "tags: #{tags.inspect}") if @debug

  #:city=>"esx", :role=>"app", :replicated =>"no"

  # Log space required
  $evm.log("info", "vm=[#{vm.name}], space required=[#{vm.provisioned_storage}]")

  host = storage = nil
  min_registered_vms = nil
  #prov.eligible_hosts.each do |h|
  cluster.hosts.each do |h|
    next unless h.power_state == "on"
    $evm.log("info", "checking host #{h.name}") if @debug
    nvms = h.vms.length
    $evm.log("info", "number of vms=#{nvms}") if @debug
    if min_registered_vms.nil? || nvms < min_registered_vms
      # If role is app && not replicated then use storage with "City" && " GP " and not VPLEX in the name.
      # If role is app && is replicated then use storage with "City" && " GP " and "VPLEX"
      # If role is db && not replicated then use storage with "City" "OS-BIN" and not VPLEX in the name.
      # If role is db && is replicated then use storage with "City" "VPLEX" and "OS-BIN".
      if tags[:role] == 'app'
        storage_type = 'GP'
      elsif tags[:role] == 'db'
        storage_type = 'OS-BIN'
      else
        raise 'role tag must be either app or db.'
      end
      $evm.log("info", "required storage role: #{storage_type}") if @debug
      storages = h.writable_storages.find_all { |s| s.free_space > vm.provisioned_storage } # Filter out storages that do not have enough free space for the Vm
      $evm.log(:info, "writable_storages.find_all: #{storages.inspect}") if @debug
      $evm.log(:info, "writable_storages names: #{storages.collect {|i| i.name }}") if @debug
      # Storage must always have the city in it.
      storages = storages.select { |s| s.name =~ /#{tags[:city]}/i }
      $evm.log(:info, "filtered storages names - city: #{storages.collect {|i| i.name }}") if @debug
      unless cluster.name == "VPLEX Unified Datacenter" # Ignore if has VPLEX or NOT in lab cluster.
        if tags[:replicated] == 'no'
          # If not replicated.
          storages = storages.select { |s| s.name !~ /VPLEX/i }
          $evm.log(:info, "filtered storages names - replicated: #{storages.collect {|i| i.name }}") if @debug
        elsif tags[:replicated] == 'yes'
          # If replicated.
          storages = storages.select { |s| s.name =~ /VPLEX/i }
          $evm.log(:info, "filtered storages names - replicated: #{storages.collect {|i| i.name }}") if @debug
        else
          raise 'replicated tag must be either yes or no.'
        end
      end
      # Storage must also have storage_type in it unless we are in the lab cluster.
      storages = storages.select { |s| s.name =~ /#{storage_type}/i } unless cluster.name == 'VPLEX Unified Datacenter'
      $evm.log(:info, "filtered storages names: #{storages.collect {|i| i.name }}") if @debug
      if storages.count == 0
        error = "No storages found that match criteria for replicated = #{tags[:replicated]} and city = #{tags[:city]}"
        error << " and storage_type = #{storage_type}" unless cluster.name == "VPLEX Unified Datacenter"
        $evm.log(:info, error)
        next
      end
      #
      s = storages.max_by(&:free_space)
      unless s.nil?
        host    = h
        storage = s
        min_registered_vms = nvms
      end
    end
  end

  $evm.log(:error, 'host not determined') if host.blank?
  $evm.log(:error, 'storage not determined') if storage.blank?
  exit MIQ_ABORT if host.blank? || storage.blank?

  # Set host and storage
  prov.set_host(host)
  prov.set_storage(storage)

  $evm.log("info", "placement: vm=[#{vm.name}] host=[#{host}] storage=[#{storage}]")

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
