#
# list_valid_networks.rb
#
# Description: List valid networks which are valid for this provision based upon other dialogs.
#
# Environment + Server Role + City determine cluster unless Cluster has been predetermined in dialog_option_1_cluster_name.
#
# To find networks:
# 1. Select the cluster as defined above.
# 2. Select a host in that cluster which is in the same city as the proposed VM based upon the first 3 letters of the host's name case insensitive.
# 3. Get the list of networks off that host.
# 3a. host.switches.each |switch|
# 3b. If host is replicated, network name must start OTV.
# 3c. check regex against each switch.lans.each |lan| and trim out non-standard names.
# 3d. Select DB or NDB for database or not database.
#
# jcutter@redhat.com March 2017.
#

begin
  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug

  # Collect info from other dialogs.
  role =  $evm.root['dialog_tag_1_role'].first.name rescue nil
  $evm.log(:info, "role: #{role}") if @debug
  city =  $evm.root['dialog_tag_1_city'].first.name rescue nil
  $evm.log(:info, "city: #{city}") if @debug
  replicated =  $evm.root['dialog_tag_1_replicated'].first.name rescue nil
  $evm.log(:info, "replicated: #{replicated}") if @debug
  cluster_name = $evm.root['dialog_option_1_cluster_name']
  $evm.log(:info, "dialog_option_1_cluster_name: #{cluster_name}")

  exit MIQ_OK if role.blank? || city.blank? || replicated.blank? || cluster_name.blank?

  cluster = $evm.vmdb(:ems_cluster).find_by_name(cluster_name)
  raise "cluster #{cluster_name} not found!" if cluster.blank?
  $evm.log(:info, "cluster: #{cluster.name}") if @debug

  # Select a host on the cluster that is in the right City.
  host = cluster.hosts.select { |host| host.name =~ /^#{city}/i }.first unless cluster.blank?
  $evm.log(:info, "host: #{host.inspect}") if @debug

  # Get all networks on all switches of the host.
  networks = host.switches.each.collect { |switch| switch.lans.each.collect { |lan| lan.name } }.flatten unless host.blank?
  $evm.log(:info, "networks: #{networks.inspect}") if @debug
  # Get all networks that match the naming convention.
  if role == 'db'
    db_prefix = 'DB'
  else
    db_prefix = 'NDB'
  end
  rep_prefix = ''
  if replicated == 'yes'
    rep_prefix = 'OTV-'
  end
  # Select network using following criteria:
  # Starts with rep_prefix
  # 4 letters and -
  # NW-
  # db_prefix and -
  # anything
  # something looking like an IP
  if cluster.name == 'VPLEX Unified Datacenter'
    regex = /^OTV[1-9]-Lab-\d{1,3}\.\d{1,3}\.\d{1,3}\./
   elsif cluster.name == 'UCS Blade Cluster'
    regex = /^StaticIP-\d{1,3}\.\d{1,3}\.\d{1,3}\./
   else
    regex = /^#{rep_prefix}[A-Za-z]{4}-NW-#{db_prefix}.*-\d{1,3}\.\d{1,3}\.\d{1,3}\./
  end
  valid_networks = networks.select { |network| network =~ regex }
  $evm.log(:info, "valid_networks: #{valid_networks}")

  dialog_values = {}
  valid_networks.each { |network| dialog_values[network] = network }
  $evm.log(:info, "values = #{dialog_values.inspect} ")

  dialog_field                  = $evm.object
  dialog_field['sort_by']       = 'value'
  dialog_field['data_type']     = 'string'
  dialog_field['values']        = dialog_values

  dialog_field

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end

