#
# list_valid_windows_networks.rb
#
# Description: List valid networks which are valid for this provision based upon other dialogs.
#
# Environment + Server Role + City determine cluster unless Cluster has been predetermined in dialog_option_1_cluster_name.
#
# jcutter@redhat.com March 2017.
#

begin
  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug

  # Start the dialog.
  dialog_field                  = $evm.object
  dialog_field['sort_by']       = 'value'
  dialog_field['data_type']     = 'string'
  dialog_field['required']      = 'true'
  dialog_values = {}

  # Determine catalog item name for deciding if this is a DB or non-DB.
  catalog_name = $evm.root['service_template'].name
  $evm.log(:info, "catalog_name: #{catalog_name}")

  # Collect info from other dialogs.
  vplex =  $evm.root['dialog_option_1_vplex']
  $evm.log(:info, "vplex: #{vplex}") if @debug
  datacenter_name = $evm.root['dialog_option_1_datacentername']
  $evm.log(:info, "datacenter_name: #{datacenter_name}")
  cluster_name = $evm.root['dialog_option_1_clustername']
  $evm.log(:info, "cluster_name: #{cluster_name}")
  vm_name = $evm.root['dialog_option_1_name']
  vm_name ||= $evm.root['dialog_sanbox_name']
  $evm.log(:info, "vm_name: #{vm_name}")

  if vplex.blank? || datacenter_name.blank? || cluster_name.blank? || vm_name.blank?
    dialog_values[''] = '<Please complete fields above>'
    $evm.log(:info, "values = #{dialog_values.inspect} ")
    dialog_field['values'] = dialog_values
    dialog_field
    exit MIQ_OK
  end

  cluster = $evm.vmdb(:ems_cluster).find_by_name(cluster_name)
  raise "cluster #{cluster_name} not found!" if cluster.blank?
  $evm.log(:info, "cluster: #{cluster.name}") if @debug
  server_prefix = vm_name[0..2]
  $evm.log(:info, "server_prefix: #{server_prefix}") if @debug
  
  if server_prefix == 'CHA' || server_prefix == 'UDX' || server_prefix == 'UDC'
    city = 'CHA'
  else
    city = 'KNX'
  end
  $evm.log(:info, "city: #{city}") if @debug

  # Select first host on the cluster that is on.
  if datacenter_name == 'Unified Datacenter T-BLOCK'
    host = cluster.hosts.select {|host| host.power_state == 'on' && host.name =~ /^#{city}/i}.first unless cluster.blank?
    $evm.log(:info, "host_name: #{host.inspect}") if @debug
  else
    host = cluster.hosts.select {|host| host.power_state == 'on'}.first unless cluster.blank?
    $evm.log(:info, "host: #{host.inspect}") if @debug
  end
  
  # Get all networks on all switches of the host.
  all_networks = host.switches.each.collect { |switch| switch.lans.each.collect { |lan| lan.name } }.flatten unless host.blank?
  $evm.log(:info, "all_networks: #{all_networks.inspect}") if @debug
  # Get all networks that have 1.2.3.4/25 or 1.2.3.x type syntax in them.
  valid_networks = all_networks.select { |network| network =~ /-\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\/\d{1,2}/ || network =~ /-\d{1,3}\.\d{1,3}\.\d{1,3}\.x{1,3}/i || /-\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\-\d{1,2}/ }
  $evm.log(:info, "valid_networks: #{valid_networks}")
  
  if vm_name[3] == 'T' || vm_name[3] == 't'
    valid_networks.each { |network| dialog_values[network] = network }
    $evm.log(:info, "values = #{dialog_values.inspect} ")
    dialog_field['values'] = dialog_values
    dialog_field
    exit MIQ_OK
  end
    
  # If we are doing VPLEX then select OTV- networks only
  if vplex == 'Yes'
    valid_networks = valid_networks.select {|network| network =~ /^OTV-|^OTV[1-9]-/i }
  else
    #valid_networks = valid_networks.select {|network| network !~ /^OTV-|^OTV[1-9]-/i}
    valid_networks = valid_networks.select {|network| network !~ /^OTV-Prod|^OTV-ACPT/i}
  end
  $evm.log(:info, "valid_networks: #{valid_networks}")

  # Determine if we are doing a DB or an NDB.
  if catalog_name == 'Windows Provisioning - SQL'
    db_prefix = 'WIN-DB'
  else
    db_prefix = 'WIN-NDB'
  end

  # If we see any with the db_prefix in them, then filter out the rest.
  if datacenter_name == 'Unified Datacenter T-BLOCK' && cluster_name == 'Management Cluster'
    filtered_networks = valid_networks.select { |network| network =~ /#{db_prefix}/i || network =~ /Tools/i}
    valid_networks = filtered_networks unless filtered_networks.blank?
    $evm.log(:info, "valid_networks: #{valid_networks}") 
    valid_networks.each { |network| dialog_values[network] = network }
    $evm.log(:info, "values = #{dialog_values.inspect} ")
    dialog_field['values'] = dialog_values
    dialog_field
    exit MIQ_OK
  else  
    filtered_networks = valid_networks.select { |network| network =~ /#{db_prefix}/i || network =~ /dmz/i }
    valid_networks = filtered_networks unless filtered_networks.blank?
    $evm.log(:info, "valid_networks: #{valid_networks}")
  end
  
  # Determine environment filter to use.
  case vm_name[3]
  when /d/i
    env_regex = 'DEV-|DEVT-'
  when /a/i
    env_regex = 'ACTP-|ACCP-|ACPT-'
  when /p/i
    env_regex = 'PROD-'
  else
    dialog_values[''] = '<Invalid Server Name>'
    $evm.log(:info, "values = #{dialog_values.inspect} ")
    dialog_field['values'] = dialog_values
    dialog_field
    exit MIQ_OK
  end

  filtered_networks = valid_networks.select { |network| network =~ /#{env_regex}/i || network =~ /dmz/i && network !~ /LEGACY/i }
  valid_networks = filtered_networks unless filtered_networks.blank?
  $evm.log(:info, "valid_networks: #{valid_networks}")

  # Add Tools and avigilon networks in to valid networks.
  #valid_networks = valid_networks + all_networks.select { |network| network =~ /tools/i }
  valid_networks = valid_networks + all_networks.select { |network| network =~ /avigilon/i }

  # Store dialog values.
  valid_networks.each { |network| dialog_values[network] = network }
  $evm.log(:info, "values = #{dialog_values.inspect} ")
  dialog_field['values'] = dialog_values
  dialog_field

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
