# Author: Brant Evans (bevans@redhat.com)
# Author: Jeffrey Cutter (jcutter@redhat.com)
# License: GPL v3
#
# Description: Add host to Ansible Inventory via API
#

begin

  require 'rest_client'
  require 'json'

  def call_api(action, api_path, payload)

    # get the server fqdn/ip and port number to send the request to
    server      = $evm.object['tower_host']
    api_version = $evm.object['api_version']
    username    = $evm.object['username']
    password    = $evm.object.decrypt('password')
    $evm.log(:info, "server: #{server}, api_version: #{api_version}, username: #{username}") if @debug

    # build the URL for the REST request
    url = "https://#{server}/api/#{api_version}/#{api_path}"

    # Tower expects the api path to end with a "/" so guarantee that it is there
    # Searches and filters don't like trailing / so exclude if includes =
    url << '/' unless url.end_with?('/') || url.include?('=')

    $evm.log(:info, "Call Tower API URL: <#{url}>") if @debug
    $evm.log(:info, "payload: #{payload.inspect}")

    # build the REST request
    params = {
        :method   => action,
        :url      => url,
        :user     => username,
        :password => password,
        :headers  => {
            :content_type  => :json,
            :accept        => :json
        },
        :timeout => 90,
        :payload => payload,
        :verify_ssl  => false
    }

    # call the Ansible REST service
    response = RestClient::Request.new(params).execute

    if response.code == 200 || response.code == 201
      response_json = JSON.parse(response)
    else
      $evm.log(:error, "Error calling Ansible Tower REST API")
      $evm.log(:info, "Response Code: <#{response.code}>  Response: <#{response.inspect}>")
    end

    #$evm.log(:info, "Ansible API Response Data: #{response_json.inspect}") if @debug

    response_json
  end

  #######
  # BEGIN HERE
  #######

  @debug = true
  $evm.log(:info, "Starting Ansible Tower REST API call")

  # Determine the Inventory ID.
  inventory_name = $evm.object['inventory_name']
  $evm.log(:info, "inventory_name: #{inventory_name}") if @debug
  payload = {}.to_json
  api_path = "inventories?name=#{CGI.escape(inventory_name)}"
  result = call_api(:get, api_path, payload)
  inventory_id = result['results'].first['id'] rescue nil
  raise "Unable to determine inventory_id from inventory_name: #{inventory_name}" if inventory_id.blank?
  $evm.log(:info, "inventory_id: #{inventory_id}") if @debug

  # Check if vm_name is already in inventory.
  prov = $evm.root["miq_provision"]
  vm_name = prov.get_option(:linux_host_name) rescue nil
  raise 'Unable to determine vm_name' if vm_name.blank?
  api_path = "inventories/#{inventory_id}/hosts?name=#{vm_name}"
  result = call_api(:get, api_path, payload)
  if result['count'] != 0
    $evm.log(:info, "VM #{vm_name} already exists in Ansible Inventory #{inventory_name}, done.")
    exit MIQ_OK
  end

  # Add the host to Ansible.
  api_path = 'hosts'
  payload = {
      :name => vm_name,
      :inventory => inventory_id,
      :enabled => true
  }.to_json

  call_api(:post, api_path, payload)

  # Verify if the vm_name is in the inventory now.
  api_path = "inventories/#{inventory_id}/hosts?name=#{vm_name}"
  result = call_api(:get, api_path, payload)
  if result['count'] == 0
    $evm.log(:info, "Failed to add #{vm_name} to Ansible Inventory #{inventory_name}.")
    exit MIQ_ABORT
  end
  $evm.log(:info, "VM #{vm_name} successfully added to Ansible inventory #{inventory_name}")

  exit MIQ_OK

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
